{% extends 'core/base.html' %}
{% load staticfiles %}
{% load compress %}

{% block headextra %}
{% compress css %}
    <link rel="stylesheet" type="text/x-scss" href="{% static 'wordpress/css/wordpress.scss' %}" />
{% endcompress %}
{% endblock %}
