@import 'core/css/variables';

html, body {
    color: $text-color;
    background-color: $background-color;
}
h1, h2, h3, h4 {
    color: $headline-color;
    background-color: $headline-background-color;
}
h1:first-child {
    margin-top: 0;
}
h2 {
    font-size: 28px;
}
h1, h2, h3 {
    line-height: 40px;
    margin-bottom: 20px;
}
h3:first-child {
    margin-top: 0;
    margin-bottom: 20px;
}
h4 {
    margin-top: 20px;
}
h5, h6 {
    font-size: medium;
    line-height: 20px;
}
form {
    margin-top: 20px;
    margin-bottom: 20px;
}

a,
.pagination > li > a {
    color: $link-color;

    &:visited {
        color: $link-color-visited;
    }
    &:hover {
        color: $link-color-hover;
    }
    &:focus {
        color: $link-color-focus;
    }
    &.btn {
        color: white;
    }
    &.fa {
        text-decoration: none;
    }
}

blockquote {
    padding: 5px 8px;
    font-size: inherit;
}

.expand {
    display: block;
    width: 100%;
}

.break {
    word-wrap: break-word;
}

/* navbar */

.navbar-default {
    background-color: $navigation-background-color;
    border-bottom: none;

    .navbar-brand,
    .navbar-nav > li > a,
    .navbar-nav > li > a:focus {
        color: $navigation-color;
        background-color: transparent;
    }
    .navbar-brand:hover,
    .navbar-nav > li > a:hover,
    .navbar-nav > .open > a,
    .navbar-nav > .open > a:focus,
    .navbar-nav > .open > a:hover {
        color: $navigation-hover-color;
        background-color: $navigation-hover-background-color;
    }

    .dropdown li.divider:first-child {
        display: none;
    }

    .dropdown-menu {
        .sub-menu {
            padding-left: 0;
            list-style-type: none;

            a {
                color: inherit;
                text-decoration: none;
            }
            li {
                padding: 1px 20px 1px 40px;
            }
            :hover {
                background-color: $navigation-dropdown-hover-background-color;
            }
        }
    }
}

/* content */

.content {

}
.sidebar {
    margin-top: 60px;
}
.page, .wide, .sidebar {
    padding-top: 90px;
    padding-bottom: 40px;
}

/* footer */

footer {
    color: $footer-color;
    background-color: $footer-background-color;

    a {
        color: $footer-link-color;

        &:visited {
            color: $footer-link-color-visited;
        }
        &:hover {
            color: $footer-link-color-hover;
        }
        &:focus {
            color: $footer-link-color-focus;
        }

        &.fa {
            text-decoration: none;
        }
    }
}

/* modals */

.modal-body {
    max-height: calc(100vh - 225px);
    overflow-y: auto;
}
.modal-body img {
    max-width: 100%;
}
.modal-body dl {
    margin-top: 5px;
    margin-bottom: 0;
}
.modal-body dl:first-child {
    margin-top: 0;
}
.modal-seperator {
    margin-left: -15px;
    margin-right: -15px;
    margin-top: 15px;
    margin-bottom: 15px;
    border-bottom: 1px solid #e5e5e5;
}

/* daiquiri-widget */

.daiquiri-widget {
    border: 1px solid $border-color;
    padding: 10px 15px;
}

.daiquiri-widget :last-child {
    margin-bottom: 0;
}

/* panel */

.panel {
    .panel-body p:last-child {
        margin-bottom: 0;
    }

    li {
        &.list-group-link {
            padding: 0;
        }
        &.list-group-link > a {
            display: block;
            padding: 10px 15px;
        }
        &.list-group-link:last-child > a {
            border-bottom-right-radius: 4px;
            border-bottom-left-radius: 4px;
        }
        &.list-group-link > a:hover {
            text-decoration: none;
            color: $panel-hover-color;
            background-color: $panel-hover-background-color;
        }
        &.list-group-link > a.active {
            text-decoration: none;
            color: $panel-active-color;
            background-color: $panel-active-background-color;
        }
    }
}

/* forms */

div.control-label {
    margin-bottom: 5px;
    font-weight: bold;
}

.daiquiri-multi-checkbox > :first-child {
    margin-top: 0;
}
.daiquiri-multi-checkbox > :last-child {
    margin-bottom: 0;
}

div.fieldset {
    margin-bottom: 20px;
}

/* misc */

.email-form label,
.connections-form label {
    display: block;
    margin: 0;
    line-height: 40px;
    border-bottom: 1px solid $modal-border-color;
}
.email-form label:first-child,
.connections-form label:first-child {
    border-top: 1px solid $modal-border-color;
}
.email-form label input,
.connections-form label input {
    margin-left: 5px;
    margin-right: 5px;
}
.email-form .email-form-buttons,
.connections-form .connections-form-buttons {
    margin-top: 10px;
}
.socialaccount_providers {
    margin: 0;
    padding: 0;
    height: 42px;
}
.socialaccount_providers li {
    float: left;
    margin-right: 10px;
    list-style: none;
}
.socialaccount_provider_name {
    line-height: 29px;
    font-weight: bold;
}

.logout-form {
    margin: 0;
}
.logout-form .btn-link {
    padding: 3px 20px;
    color: $navigation-dropdown-color;
    display: block;
    width: 100%;
    text-align: left;
    border: none;
    clear: both;
    font-weight: 400;
    line-height: 1.42857143;
    white-space: nowrap;
}
.logout-form .btn-link:hover {
    color: $navigation-dropdown-hover-color;
    background-color: $navigation-dropdown-hover-background-color;
    text-decoration: none;
}
.logout-form .btn-link:focus {
    color: $navigation-dropdown-hover-color;
    background-color: $navigation-dropdown-hover-background-color;
    text-decoration: none;
    outline: none;
}
