{% load i18n %}
{% load core_tags %}

    <a href="" class="fa fa-eye" title="{% trans 'Show user details' %}"
        ng-click="service.modal('show-user-modal', $index)">
    </a>
    <a href="" class="fa fa-pencil" title="{% trans 'Update user' %}"
        ng-click="service.modal('update-user-modal', $index)">
    </a>

    <span ng-show="{$ row.id $} != {{ request.user.id }} && !row.is_pending">
        <a href="" class="fa fa-times-circle"
            ng-show="row.user.is_active" title="{% trans 'Disable user' %}"
            ng-click="service.modal('disable-user-modal', $index)">
        </a>
        <a href="" class="fa fa-check-circle"
            ng-show="!row.user.is_active" title="{% trans 'Enable user' %}"
            ng-click="service.modal('enable-user-modal', $index)">
        </a>
    </span>

    <span ng-show="row.is_pending">
        {% if settings.AUTH_WORKFLOW == 'confirmation' %}
        <span ng-show="!row.is_confirmed">
            <a href="" class="fa fa-check" title="{% trans 'Confirm the user' %}"
                ng-click="service.modal('confirm-user-modal', $index)">
            </a>
            <a href="" class="fa fa-times" title="{% trans 'Reject the user' %}"
                ng-click="service.modal('reject-user-modal', $index)">
            </a>
        </span>
        {% endif %}

        {% if settings.AUTH_WORKFLOW == 'activation' %}
        <a href="" class="fa fa-check" title="{% trans 'Activate the user' %}"
            ng-click="service.modal('activate-user-modal', $index)">
        </a>
        <a href="" class="fa fa-times" title="{% trans 'Reject the user' %}"
            ng-click="service.modal('reject-user-modal', $index)">
        </a>
        {% endif %}
    </span>

    {% if user.is_staff %}
    <div class="dropdown">
        <a class="dropdown-toggle" type="button" id="users-options"
            data-toggle="dropdown" aria-haspopup="true" aria-expanded="true"
            title="{% trans 'Show options' %}">
            <span class="fa fa-ellipsis-h"></span>
        </a>
        <ul class="dropdown-menu dropdown-menu-right" aria-labelledby="users-options">
            {% if settings.AUTH_WORKFLOW == 'confirmation' %}
            <li>
                <a href="" ng-show="row.is_pending && row.is_confirmed" ng-click="service.modal('activate-user-modal', $index)">
                    {% trans 'Activate the user' %}
                </a>
            <li>
            {% endif %}
            <li>
                <a href="{{ user_admin_url }}">{% trans 'User Admin' %}</a>
            </li>
            <li>
                <a href="{{ profile_admin_url }}">{% trans 'Profile Admin' %}</a>
            </li>
        </ul>
    </div>

    {% endif %}
