<!DOCTYPE html>
<html>

<head>
    {% include 'core/base_head.html' %}
    {% block headextra %}
    {% endblock %}
</head>

<body {% block bodyargs %}{% endblock %}>

<!-- begin navigation -->
    {% include 'core/base_navigation.html' %}
<!-- end navigation -->

<!-- begin content -->

    <div class="content">
        {% block content %}
        <!-- content placeholder -->
        {% endblock %}
    </div>

<!-- end content -->

<!-- begin footer -->
    {% include 'core/base_footer.html' %}
<!-- end footer -->

{% if not debug %}
<!-- begin analytics -->
    {% include 'core/base_analytics.html' %}
<!-- end analytics -->
{% endif %}
</body>
</html>
