
<div class="form-group field-{{ field }}">
    <label for="id_{{ field }}">{{ label }}</label>
    <input class="form-control"
           id="id_{{ field }}"
           name="{{ field }}"
           type="text"
           ng-model="{{ ng_model }}" />

    <div class="help-block error">
    </div>

    <div class="help-block info">
    </div>
</div>
