
<div class="checkbox field-{{ field }}">
    <label>
        <input type="checkbox"
               name="{{ field }}"
               ng-model="{{ ng_model }}" />
        {{ label }}
    </label>
</div>
