{% load i18n %}

<p>
    {% trans 'Please specify a coordinate in right ascension (RA) and declination (DEC) and a cone radius. The query will result in objects that have coordinates in the search cone.' %}
</p>

<div class="row fieldset">
    <div class="col-md-4">
        <formgroup
            data-type="text"
            data-label="{% trans 'RA (degrees)' %}"
            data-model="service.values.cone.ra"
            data-errors="service.errors.cone.ra">
        </formgroup>
    </div>
    <div class="col-md-4">
        <formgroup
            data-type="text"
            data-label="{% trans 'DE (degrees)' %}"
            data-model="service.values.cone.de"
            data-errors="service.errors.cone.de">
        </formgroup>
    </div>
    <div class="col-md-4">
        <formgroup
            data-type="text"
            data-label="{% trans 'Radius (arcsec)' %}"
            data-model="service.values.cone.radius"
            data-errors="service.errors.cone.radius">
        </formgroup>
    </div>
</div>

<div class="row">
    <formgroup
        class="col-md-9"
        data-type="text"
        data-label="{% trans 'Table name' %}"
        data-model="service.forms.cone.values.table_name"
        data-errors="service.forms.cone.errors.table_name">
    </formgroup>
    <formgroup
        class="col-md-3"
        data-type="select"
        data-label="{% trans 'Queue' %}"
        data-model="service.active_queue"
        data-options="service.queues"
        data-options-label="text">
    </formgroup>
</div>

<button class="btn btn-primary" ng-click="service.forms.cone.submit()">
    {% trans 'Submit new cone search' %}
</button>

<button class="btn" ng-click="">
    {% trans 'Clear input window' %}
</button>
